@@ -1,3 +1,4 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
@@ -30,11 +31,13 @@ def find_version(*file_paths):
 # -- General configuration ----------------------------------------------------
 
 extensions = [
+    "notfound.extension",
     "sphinx.ext.autodoc",
     "sphinx.ext.autodoc.typehints",
     "sphinx.ext.doctest",
     "sphinx.ext.intersphinx",
     "sphinx.ext.viewcode",
+    "sphinxcontrib.mermaid",
 ]
 
 # Add any paths that contain templates here, relative to this directory.
@@ -72,9 +75,7 @@ nitpick_ignore = [
     ("py:class", "structlog.threadlocal.TLLogger"),
     ("py:class", "TextIO"),
     ("py:class", "TLLogger"),
-    ("py:class", "Token"),
     ("py:class", "traceback"),
-    ("py:class", "contextvars.Token"),
     ("py:class", "structlog._base.BoundLoggerBase"),
     ("py:class", "structlog.dev._Styles"),
     ("py:class", "structlog.types.EventDict"),
