@@ -1,8 +1,8 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-
 import datetime
 import json
 import pickle
@@ -274,6 +274,22 @@ class TestTimeStamper:
             None, None, {}
         )
 
+    @pytest.mark.parametrize(
+        ("utc", "expect"),
+        [
+            (True, "1980-03-25T16:00:00Z"),
+            (False, "1980-03-25T17:00:00"),
+        ],
+    )
+    def test_apply_freezegun_after_instantiation(self, utc, expect):
+        """
+        Instantiate TimeStamper after mocking datetime
+        """
+        ts = TimeStamper(fmt="iso", utc=utc)
+        with freeze_time("1980-03-25 16:00:00", tz_offset=1):
+            d = ts(None, None, {})
+            assert expect == d["timestamp"]
+
 
 class TestFormatExcInfo:
     @pytest.mark.parametrize("ei", [False, None, ""])
